/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

package com.google.code.joliratools.logdb.parser;

/**
 * The callback interface used for processing data identified by the parser.
 * 
 * @author jfk
 * @since 1.0
 */
public interface Record {
    /**
     * The type of record.
     */
    public static enum Type {
        /**
         * The record represents data that was correctly recognized.
         */
        RECOGNIZED,
        /**
         * The records represents a valid entry (header elements where found), which was not recognized by the parser.
         */
        UNRECOGNIZED,
        /**
         * An entry in the input stream was not valid
         */
        JUNK
    }

    /**
     * @return the class name found in the header of the record. This call returns {@literal null} if {@link #getType()}
     *         returns {@link Type#JUNK}.
     */
    public String getClazz();

    /**
     * @return a unique id if the type of record being recognized. This call returns {@literal null} if
     *         {@link #getType()} returns {@link Type#JUNK} or {@link Type#UNRECOGNIZED}.
     */
    public String getId();

    /**
     * @return the level found in the header of the record. This call returns {@literal null} if {@link #getType()}
     *         returns {@link Type#JUNK}.
     */
    public String getLevel();

    /**
     * @return the line number of the first line in the record.
     */
    public int getLineNumber();

    /**
     * @return the method name found in the header of the record. This call returns {@literal null} if
     *         {@link #getType()} returns {@link Type#JUNK}.
     */
    public String getMethod();

    /**
     * @return the row data found.
     */
    public String getRaw();

    /**
     * @return the time stamp of the record. This call returns {@literal null} if {@link #getType()} returns
     *         {@link Type#JUNK}.
     */
    public long getTimestamp();

    /**
     * @return the type of record
     */
    Type getType();

    /**
     * @return the variables found in the data. This call returns {@literal null} if {@link #getType()} returns
     *         {@link Type#JUNK} or {@link Type#UNRECOGNIZED}.
     */
    public String[] getVariables();
}
