/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */
package com.google.code.joliratools.logdb.reader;

import static com.google.code.joliratools.logdb.parser.Record.Type.RECOGNIZED;
import static com.google.code.joliratools.logdb.parser.Record.Type.UNRECOGNIZED;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import com.google.code.joliratools.logdb.parser.Parser;
import com.google.code.joliratools.logdb.parser.Record;
import com.google.code.joliratools.logdb.parser.Record.Type;

/**
 * @author jfk
 * @date Sep 16, 2010 7:41:31 AM
 * @since 1.0
 */
public abstract class LogDigester implements Reader {
    private final class EntryWrapper implements Entry {
        private final Record record;
        private Map<String, String> cachedVars = null;
        private Type cachedType = null;
        private IOException e = null;

        EntryWrapper(final Record record) {
            this.record = record;
        }

        @Override
        public String getClazz() {
            return record.getClazz();
        }

        @Override
        public String getId() {
            return record.getId();
        }

        @Override
        public String getLevel() {
            return record.getLevel();
        }

        @Override
        public int getLineNumber() {
            return record.getLineNumber();
        }

        @Override
        public String getMethod() {
            return record.getMethod();
        }

        @Override
        public String getRaw() {
            return record.getRaw();
        }

        @Override
        public long getTimestamp() {
            return record.getTimestamp();
        }

        @Override
        public Type getType() {
            parseVariables();

            return cachedType;
        }

        @Override
        public Map<String, String> getVariables() throws IOException {
            parseVariables();

            final IOException _e = e;

            if (_e != null) {
                throw _e;
            }

            return cachedVars;
        }

        private synchronized void parseVariables() {
            if (cachedType != null) {
                return;
            }

            final Type type = record.getType();

            if (!RECOGNIZED.equals(type)) {
                cachedType = type;
                cachedVars = new HashMap<String, String>();
                return;
            }

            Map<String, String> vars = null;

            try {
                vars = processVariables(record);
            } catch (final IOException _e) {
                e = _e;
            }

            cachedType = vars != null ? type : UNRECOGNIZED;
            cachedVars = vars != null ? vars : new HashMap<String, String>();
        }

        @Override
        public String toString() {
            final StringBuilder builder = new StringBuilder();

            builder.append("LogDigester#Entry [record=");
            builder.append(record);

            if (cachedVars != null) {
                builder.append(", ");
                builder.append("cachedVars=");
                builder.append(cachedVars);
            }

            return builder.toString();
        }
    }

    /**
     * @see Reader#getNext(Parser)
     */
    @Override
    public Entry getNext(final Parser parser) throws IOException {
        final Record record = parser.getNext();

        if (record == null) {
            return null;
        }

        return new EntryWrapper(record);
    }

    /**
     * @param record
     *            the record to be processed
     * @return the variables by name.
     * @throws IOException
     *             reading failed
     */
    protected abstract Map<String, String> processVariables(Record record) throws IOException;
}
